class URLs:
    def __init__(self):
        self.base_url = "https://api.mojang.com"

        # Player
        self.player_profile_url = "/users/profiles/minecraft/{}"

    def player_profile(self, player_name):
        return self.base_url + self.player_profile_url.format(player_name)


import novauniverse as nova

player = nova.Player(player_name="THEGOLDENPRO")

session = nova.Session(session_id="95")
print(session.game.code_name)

#print(player.last_join)

player = nova.Player(player_name="THEGOLDENPRO")

print(player.name)
print(player.sessions[0].game.name)
print(player.is_online)


import requests
import json

from .. import info

api_name = "NOVA API"
headers = {'User-Agent': str(info.PACKAGE_NAME)}
error_format = f"\u001b[31m{api_name}" + ": [{}] {}\u001b[0m"

class NovaAPIError(Exception):
    pass

def request(url:str): #Makes request to api then returns data in json
    #Make request.
    response = requests.get(url, headers=headers)
    data:dict = response.json()
    
    try: #Nova Universe API error catcher.
        if data["success"] == False:
            error = data["error"]
            message = data["message"]

            raise NovaAPIError(error_format.format(error, message))
    except KeyError:
        pass

    return data["data"]


from setuptools import setup, find_packages
 
classifiers = [
  'Operating System :: Microsoft :: Windows :: Windows 10',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3.8'
]
 
setup(
  name='novauniverse',
  version='1.0a4',
  description='A modern API wrapper for the minecraft server Nova Universe written in Python.', 
  long_description=open('README.txt').read(), 
  url='https://novauniverse.net/', 
  project_urls={"Bug Tracker": "https://github.com/NovaUniverse/novauniverse.py/issues"}, 
  author='Dev Goldy', 
  author_email='jassim7256@gmail.com', 
  license='MIT', 
  classifiers=classifiers, 
  keywords=['novauniverse', 'minecraft novauniverse', 'nova universe'], 
  packages=find_packages(), 
  install_requires=["requests"],
  python_requires=">=3.7"
)


import datetime

class basic_player(object):
    def __init__(self, id:int, uuid:str, username:str):
        self.id_ = id
        self.uuid_ = uuid
        self.username_ = username

    @property
    def id(self): #Returns Nova Universe id of player.
        id:str = self.id_; return id

    @property
    def uuid(self): #Returns Mojang uuid of player.
        uuid:str = self.uuid_; return uuid

    @property
    def username(self):
        username:str = self.username_; return username

    @property
    def name(self): #Alias of "player().username"
        return self.username

class player(basic_player): #Player Object
    def __init__(self, id:int, uuid:str, username:str, first_join:dict, last_join:dict, is_online:int, sessions:list):
        super().__init__(id, uuid, username)
        self.first_join_ = first_join
        self.last_join_ = last_join
        self.is_online_ = is_online
        self.sessions_ = sessions

    from . import _session_

    class player_session(_session_.basic_session): #Class for player's game sessions.
        def __init__(self, game: dict, session_id: int, metadata: str, total_places: int, player_placement:int, timestamp: dict):
            super().__init__(game, session_id, metadata, total_places, timestamp)

            self.player_placement_ = player_placement

        @property
        def player_placement(self):
            player_placement:int = self.player_placement_; return player_placement

    @property
    def first_join(self):
        datetime_object:datetime.datetime = datetime.datetime.strptime(self.first_join_, '%Y-%m-%d %H:%M:%S.%f')
        return datetime_object

    @property
    def last_join(self):
        datetime_object:datetime.datetime = datetime.datetime.strptime(self.last_join_, '%Y-%m-%d %H:%M:%S.%f')
        return datetime_object

    @property
    def is_online(self): #Returns True/False if the player is currently present on the network.
        online:int = self.is_online_

        if online == 1: return True
        else: return False

    @property
    def sessions(self): #Returns list of game sessions as session objects.
        sessions_list = []
        for session_ in self.sessions_:
            sessions_list.append(self.player_session(session_["game"], session_["session_id"], session_["metadata"], session_["total_places"], 
            session_["player_placement"], session_["timestamp"]))

        return sessions_list

    class _utls:
        def __init__(self):
            pass

        def player_name_to_uuid(self, player_name:str):
            from ..endpoints import mojang
            from ..APIs import mojang as mojang_api

            short_uuid:str = (mojang_api.request(mojang.URLs().player_profile(player_name)))["id"]; return self.short_to_full_uuid(short_uuid)

        def short_to_full_uuid(self, short_uuid:str): #Converts short uuid to full uuid.
            return short_uuid[:8] + "-" + short_uuid[8:12] +  "-" + short_uuid[12:16] +  "-" + short_uuid[16:20] +  "-" + short_uuid[20:]


from .endpoints import nova, mojang
from .APIs import nova as nova_api, mojang as mojang_api

from .objects import player as player_, session as session_

def Player(player_name:str=None, player_uuid:str=None): #Find a player and return data as player object.
    if player_uuid == None:
        if not player_name == None:
            player_uuid = player_name_to_uuid(player_name)
        else:
            raise Exception("Neither player name or player uuid was passed through the 'player object'. Please pass one of them in.")

    url = nova.URLs().player_stats(player_uuid)
    plain_data = nova_api.request(url)

    return player_(plain_data["id"], plain_data["uuid"], plain_data["username"], plain_data["first_join_timestamp"]["date"], plain_data["last_join_timestamp"]["date"], plain_data["is_online"], plain_data["sessions"])

def Session(session_id:str): #Find a game session and return data as a full session object.
    url = nova.URLs().session_stats(session_id)
    plain_data = nova_api.request(url)

    return session_(plain_data["game"], plain_data["session_id"], plain_data["metadata"], plain_data["total_places"], plain_data["timestamp"]["date"], plain_data["players"])



def player_name_to_uuid(player_name:str):
    short_uuid:str = (mojang_api.request(mojang.URLs().player_profile(player_name)))["id"]; return short_to_full_uuid(short_uuid)

def short_to_full_uuid(short_uuid:str): #Converts short uuid to full uuid.
    return short_uuid[:8] + "-" + short_uuid[8:12] +  "-" + short_uuid[12:16] +  "-" + short_uuid[16:20] +  "-" + short_uuid[20:]